<?php
/**
 * myfreepersonals.com search changes
 */

elgg_register_event_handler('init', 'system', 'mfp_search_init');

/**
 * Init plugin
 */
function mfp_search_init() {

	// search css
	elgg_extend_view('elgg.css', 'css/mfp_search.css');
	
	elgg_register_css('mfp_search/jquerymobile', 'mod/mfp_search/views/default/js/mfp_search/jquerymobile.css');

	// override the default search
	elgg_register_page_handler('mfp_search', 'mfp_search_page_handler');
	elgg_register_page_handler('search', 'mfp_search_results_page_handler');

	// return a custom search type 'mfp_search'
	elgg_register_plugin_hook_handler('search_types', 'get_types', 'mfp_search_types');

	// do the search
	elgg_register_plugin_hook_handler('search', 'mfp_search', 'mfp_search_hook');

	// override search exclusion
	elgg_register_plugin_hook_handler('robots.txt', 'site', 'mfp_exclude_robots');

	// allow a search on email address.
	elgg_extend_view('admin/user_opt/search', 'mfp_search/admin/user_opt/search_by_email');
	elgg_register_page_handler('mfp_search_email', 'mfp_search_email');

	// widgets
	// recently active matches
	elgg_register_widget_type('mfp_search_newest_matches',
		elgg_echo('mfp_search:widget:newest_matches'),
		elgg_echo('mfp_search:widget:newest_matches:desc'),
		array('dashboard'));

	// recently active members
	elgg_register_widget_type('mfp_search_recently_active_matches',
		elgg_echo('mfp_search:widget:recently_active_matches'),
		elgg_echo('mfp_search:widget:recently_active_matches:desc'),
		array('dashboard'));

	// add a page menu items
	$item = new ElggMenuItem('mfp_search', 'Matches', '/mfp_search');
	elgg_register_menu_item('site', $item);
	$item = new ElggMenuItem('home', 'Search', '/');
	elgg_register_menu_item('site', $item);

}

/**
 * Register for custom search
 */
function mfp_search_types($hook, $type, $value, $params) {
	$value[] = 'mfp_search';

	return $value;
}

/**
 * Show custom search
 *
 */
function mfp_search_page_handler($page) {

	$parms = array();
	// need to specify this for the page handler overrides
	$parms['mfp_profile_search'] = 1;
	$parms['mfp_matches'] = 1;
	$parms['pics_only'] = 1;
	$parms['sort'] = 'newest';

	// some we can detect
	$search_fields = array(
		'gender',
		'seeking_gender',
		'match_minimum_age',
		'match_maximum_age',
		'seeking_relationship',
		'zip_code',
		'match_maximum_distance'
	);

	$user = elgg_get_logged_in_user_entity();
	
	if ($user && strtolower($user->match_location_default) == 'yes') {
		$search_fields[] = 'state_nation';
	}
	
	if ($user) {
		foreach ($search_fields as $name) {
			$value = $user->$name;
			if (!$value || empty($value)) {
				$value = null;
			}
			$parms[$name] = $value;
		}
		if (!is_array($parms['seeking_gender'])) {
			$parms['seeking_gender'] = array($parms['seeking_gender']);
		}
		else {
			$parms['seeking_gender'] = array_filter($parms['seeking_gender']);
		}
		if (!is_array($parms['seeking_relationship'])) {
			$parms['seeking_relationship'] = array($parms['seeking_relationship']);
		}
	}

	$query = http_build_query($parms);
	$url = elgg_get_site_url() . "search/mfp_search?$query";
	forward($url);

	return true;
}

/**
 * Show custom results.
 */
function mfp_search_results_page_handler($page) {

	$item = elgg_get_menu_item('site', 'mfp_search');
	if ($item) {
		$item->setSelected();
	}

	if (get_input('mfp_profile_search') == 1) {
		include_once 'pages/search_results.php';
	} else {
		search_page_handler($page);
	}
	
	return true;
}

/**
 * Search user metadata based upon passed params
 */
function mfp_search_hook() {
	$db_prefix = get_config('dbprefix');

	// denormalizing the metastrings
	// should help with performance
	$metastrings = array('gender', 'seeking_gender', 'seeking_relationship', 'birthday', 'state_nation');
	$metastring_ids = array();

	foreach ($metastrings as $string) {
		$id = elgg_get_metastring_id($string, false);

		if ($string && $id) {
			$metastring_ids[$string] = mfp_search_make_in_clause($id);
		}
	}

	$results = mfp_get_search_params();

	// denormalize result metastrings
	$results_ms_ids = array();
	foreach ($results as $name => $value) {
		$ms_ids = array();

		// some of these could have multiple values
		if (is_array($value)) {
			$sub_ids = array();
			foreach ($value as $subvalue) {
				$tmp_id = elgg_get_metastring_id($subvalue, false);
				if (is_array($tmp_id)) {
					$ms_ids = array_merge($ms_ids, $tmp_id);
				} else {
					$ms_ids[] = $tmp_id;
				}
			}
		} else {
			$ms_ids = elgg_get_metastring_id($value, false);
		}

		$results_ms_ids[$name] = mfp_search_make_in_clause($ms_ids);
	}

	$wheres = array();
	$joins = array();
	$md_pairs = array();

	// never show yourself if you're logged in
	$user_guid = elgg_get_logged_in_user_guid();

	if ($user_guid) {
		$wheres[] = "e.guid != $user_guid";
	}

	// match searcher's gender against sought user's seeking gender
	$joins[] = "JOIN {$db_prefix}metadata searcher_gender on e.guid = searcher_gender.entity_guid";
	$wheres[] = "(searcher_gender.name_id IN ({$metastring_ids['seeking_gender']}) and searcher_gender.value_id IN ({$results_ms_ids['gender']}))";

	// match searcher's seeking gender against sought user's gender
	if ($results['seeking_gender']) {
		$joins[] = "JOIN {$db_prefix}metadata seeking_gender on e.guid = seeking_gender.entity_guid";
		$wheres[] = "(seeking_gender.name_id IN ({$metastring_ids['gender']}) and seeking_gender.value_id IN ({$results_ms_ids['seeking_gender']}))";
	}

	// match searcher's seeking relationship against sought user's seeking relationship
	if ($results['seeking_relationship']) {
		$joins[] = "JOIN {$db_prefix}metadata seeking_relationship on e.guid = seeking_relationship.entity_guid";
		$wheres[] = "(seeking_relationship.name_id IN ({$metastring_ids['seeking_relationship']}) and seeking_relationship.value_id IN ({$results_ms_ids['seeking_relationship']}))";
	}

	// zip code search
	if ($results['zip_code'] && $results['match_maximum_distance']) {

		// If there is no zip code match this returns nothing, making the
		// call to mfp_get_entities_from_location() return nothing.
		// This is the correct behavior.
		$lat_long = mfp_get_lat_long_from_geocode_cache($results['zip_code']);

		$joins[] = "JOIN {$db_prefix}users_geocode ug on e.guid = ug.guid";

		// doing this in PHP to help the poor db server out.
		$lat = (float)$lat_long['latitude'];
		$long = (float)$lat_long['longitude'];
		$distance = MILE * (float)$results['match_maximum_distance'];

		$lat_min = $lat - $distance;
		$lat_max = $lat + $distance;
		$long_min = $long - $distance;
		$long_max = $long + $distance;

		$wheres[] = "ug.latitude BETWEEN $lat_min AND $lat_max";
		$wheres[] = "ug.longitude BETWEEN $long_min AND $long_max";
	}

	if ($results['state_nation']) {
		$joins[] = "JOIN {$db_prefix}metadata state_nation on e.guid = state_nation.entity_guid";
		$wheres[] = "(state_nation.name_id IN ({$metastring_ids['state_nation']}) and state_nation.value_id IN ({$results_ms_ids['state_nation']}))";
	}

	// age not required?
	// don't need to check if the values exist here because we're not using an in clause
	if ($results['match_maximum_age']) {
		// can't denormalize birthday.
		$md_pairs[] = array(
			'name' => 'birthday',
			'operand' => '>',
			'value' => $results['match_maximum_age']
		);
	}

	if ($results['match_minimum_age']) {
		$md_pairs[] = array(
			'name' => 'birthday',
			'operand' => '<',
			'value' => $results['match_minimum_age']
		);
	}

	// join for the icontime to grab only profiles with pics.
	if ($results['pics_only']) {
		$metadata_names = 'icontime';
	} else {
		$metadata_names = ELGG_ENTITIES_ANY_VALUE;
	}

	// add the domain match
	// this is saved as a profile field under "about my match"
	if (!elgg_in_context('widget')) {
		$user = elgg_get_logged_in_user_entity();
		$domains = mfp_get_profile_field_attr('domain', 'options');

		if ($user) {
			$match = $user->match_domain;
			$domain = $user->domain;

			if ($match == 'Yes' && in_array($domain, $domains)) {
				$md_pairs[] = array(
					'name' => 'domain',
					'value' => $domain
				);
			}
		} elseif (isset($_SESSION['mfp_theme_switch_referer'])) {
			// referred users only see from their referred domains.
			$domain = $_SESSION['mfp_theme_switch_referer'];
			if (in_array($domain, $domains)) {
				$md_pairs[] = array(
					'name' => 'domain',
					'value' => $domain
				);
			}
		}
	}
	// decide the order
	$sort = (isset($results['sort'])) ? $results['sort'] : 'newest';

	switch ($sort) {
		default:
		case 'newest':
			$order_by = 'e.time_created desc';
			break;

		case 'last_visit':
			$joins[] = "JOIN {$db_prefix}users_entity ue on e.guid = ue.guid";
			$order_by = 'ue.last_login desc';
			break;
	}

	// check for text search
	if ($results['text']) {
		$options = mfp_get_user_text_search_options($results['text'], null, null);
		$wheres = array_merge($wheres, $options['wheres']);
		$joins = array_merge($joins, $options['joins']);
	}

	$options = array(
		'type' => 'user',
		'subtype' => ELGG_ENTITIES_ANY_VALUE,
		'offset' => get_input('offset', 0),
		'limit' => get_input('limit', 10),

		// sorting
		'order_by' => $order_by,

		// birthday
		'metadata_name_value_pairs' => $md_pairs,

		// pics only
		'metadata_names' => $metadata_names,

		// all the other attributes
		'wheres' => $wheres,
		'joins' => $joins
	);

	$options['count'] = TRUE;

	// We're disabling access controls for the search because we want to find everyone
	// who matches. Controls are re-enabled for the display, so if a user sets his info to logged
	// in it displays "unknown" on the actual results.
	$ia = elgg_set_ignore_access(true);

	$count = elgg_get_entities_from_metadata($options);

	// no need to continue if nothing here.
	if (!$count) {
		return array('entities' => array(), 'count' => $count);
	}

	$options['count'] = FALSE;
	$entities = elgg_get_entities_from_metadata($options);

	elgg_set_ignore_access($ia);

	return array(
		'entities' => $entities,
		'count' => $count,
	);
}

/**
 * Return search params from get
 */
function mfp_get_search_params() {
	$params = array(
		'gender',
		'seeking_gender',
		'match_minimum_age',
		'match_maximum_age',
		'seeking_relationship',
		'pics_only',
		'sort',
		'zip_code',
		'match_maximum_distance',
		'q',
		'tags',
		'text',
        'member_query',
		'state_nation'
	);


	$return = array();

	foreach ($params as $param) {
		$value = get_input($param, '');

		switch ($param) {
			// normalize this to 00:00:00
			case 'match_maximum_age':
				if ($value > 17) {
					$value = $value+1;
				}
				$value = strtotime("-$value years");
				$time = mktime($hour = 0, $minute = 0, $second = 0,
					date("n", $value), date("j", $value), date("Y", $value));
				
				$value = $time;
				break;
			case 'match_minimum_age':
				$value = strtotime("-$value years");
				$time = mktime($hour = 0, $minute = 0, $second = 0,
					date("n", $value), date("j", $value), date("Y", $value));
				$value = $time;
				break;
			case 'zip_code':
				if (!preg_match('/^[0-9]{5}$/', $value)) {
					$value = '';
				}
				break;
		}

		$return[$param] = $value;
	}

	// fix the automatic tag linking
	if (isset($return['q'])) {
		$return['tags'] = $return['q'];
	}

	return $return;
}

/**
 * Returns comma separated IDs of metastrings for use in a where clause.
 *
 * @param string $string
 */
function mfp_get_metastring_id($string) {
	if (!$string) {
		return '';
	}

	//@mfpupdate
	$id = elgg_get_metastring_id($string, false);

	// if there are multiple strings in different cases pull them out.
	if ($id && is_array($id)) {
		$id = implode(',', $id);
	}

	return $id;
}

/**
 * Returns an attribute of a profile manager's field.
 *
 * @param string The value
 * @param mixed $attr the value to get: options or type.
 * @return mixed
 */
function mfp_get_profile_field_attr($field_name, $attr = null) {
	static $CACHE = array();

	if (isset($CACHE[$field_name])) {
		$field = $CACHE[$field_name];
	} else {
		$options = array(
			"type" => "object",
			"subtype" => CUSTOM_PROFILE_FIELDS_PROFILE_SUBTYPE,
			"limit" => 0,
			"owner_guid" => elgg_get_site_entity()->guid,
			'metadata_name' => 'metadata_name',
			'metadata_value' => $field_name
		);

		$field = elgg_get_entities_from_metadata($options);

		$CACHE[$field_name] = $field;
	}
	
	if ($field) {
		if ($attr) {
			switch ($attr) {
				case 'options':
					return $field[0]->getOptions();
					break;

				case 'type':
					return $field[0]->metadata_type;
					break;

				default:
					break;
			}
		} else {
			return $field[0];
		}
	}

	return false;
}

/**
 * Adds a quote to each variable
 *
 * @param unknown_type $array
 * @param unknown_type $quote
 */
function mfp_search_make_in_clause($array, $escape = true, $quote = "'") {
	if (!is_array($array)) {
		$array = array($array);
	}

	$r = array();
	foreach ($array as $i => $v) {
		$v = sanitize_string($v);
		$r[$i] = $quote .  $v . $quote;
	}

	return implode(',', $r);
}

/**
 * Return users by email.
 *
 * @param array $page
 */
function mfp_search_email($page) {
	admin_gatekeeper();
	$email = get_input('email');
	$user = get_user_by_email($email);

	$title = elgg_echo('admin:user');

	if ($user) {
		set_context('search');

		$results = elgg_view_entity_list($user, 1, 0,
			1, false, false, false);
	} else {
		$results = elgg_echo('notfound');
		$results = "<div class=\"search_listing mfp_search_block\">$results</div>";
	}

	elgg_set_context('admin');

	// Display main admin menu
	$layout = elgg_view_layout('admin', array(
		'title' => $title,
		'content' => elgg_view('admin/user') . $results
	));
	
	echo elgg_view_page($title, $layout);
}

/**
 * Allow robots to index search pages
 *
 * @param string $hook Hook name
 * @param string $type Hook type
 * @param string $text robots.txt content for plugins
 * @return string
 */
function mfp_exclude_robots($hook, $type, $text) {
	$text = <<<TEXT
User-agent: *

TEXT;

	return $text;
}
