<?php

elgg_require_js('mfp_search');

$defaults = array(
	'gender' 				=> isset($vars['defaults']['gender']) ? $vars['defaults']['gender'] : 'Female',
	'seeking_gender' 		=> isset($vars['defaults']['seeking_gender']) ? $vars['defaults']['seeking_gender'] : 'Male',
	'match_minimum_age'		=> isset($vars['defaults']['match_minimum_age']) ? $vars['defaults']['match_minimum_age'] : '18',
	'match_maximum_age'		=> isset($vars['defaults']['match_maximum_age']) ? $vars['defaults']['match_maximum_age'] : '35',
	'seeking_relationship'	=> isset($vars['defaults']['seeking_relationship']) ? $vars['defaults']['seeking_relationship'] : '',
	'state_nation' 			=> isset($vars['defaults']['state_nation']) ? $vars['defaults']['state_nation'] : '',
	'sort' 					=> isset($vars['defaults']['sort']) ? $vars['defaults']['sort'] : 'newest',
	'zip_code'				=> isset($vars['defaults']['zip_code']) ? $vars['defaults']['zip_code'] : '',
	'match_maximum_distance'=> isset($vars['defaults']['match_maximum_distance']) ? $vars['defaults']['match_maximum_distance'] : '',
	'text'                  => isset($vars['defaults']['text']) ? $vars['defaults']['text'] : '',
	'pics_only'             => isset($vars['defaults']['pics_only']) ? $vars['defaults']['pics_only'] : '',
);

foreach ($defaults as $k => $d) {
	if (is_array($d)) {
		$defaults[$k] = array_values(array_filter($d));
	}
}

// @todo do you want to be able to override this?
$gender_label = elgg_echo('mfp:search:i_am');
$gender_type = mfp_get_profile_field_attr('gender', 'type');
$gender_options = mfp_get_profile_field_attr('gender', 'options');
$gender = elgg_view("input/$gender_type", array(
	'name' => 'gender',
	'value' => $defaults['gender'],
	'options' => $gender_options
));

$seeking_gender_label = elgg_echo('mfp:search:seeking');
$seeking_gender_type = mfp_get_profile_field_attr('seeking_gender', 'type');
$seeking_gender_options = mfp_get_profile_field_attr('seeking_gender', 'options');

$seeking_gender = elgg_view("input/$seeking_gender_type", array(
	'id' => 'seeking_gender',
	'name' => 'seeking_gender',
	'value' => $defaults['seeking_gender'],
	'options' => $seeking_gender_options
));

$age_label = elgg_echo('mfp:search:age');
$match_minimum_age_type = mfp_get_profile_field_attr('match_minimum_age', 'type');
$match_minimum_age_options = mfp_get_profile_field_attr('match_minimum_age', 'options');
$match_minimum_age = elgg_view("input/$match_minimum_age_type", array(
	'id' => 'match_minimum_age',
	'name' => 'match_minimum_age',
	'value' => $defaults['match_minimum_age'],
	'options' => $match_minimum_age_options
));

$match_maximum_age_type = mfp_get_profile_field_attr('match_maximum_age', 'type');
$match_maximum_age_options = mfp_get_profile_field_attr('match_maximum_age', 'options');
$match_maximum_age = elgg_view("input/$match_maximum_age_type", array(
	'id' => 'match_maximum_age',
	'name' => 'match_maximum_age',
	'value' => $defaults['match_maximum_age'],
	'options' => $match_maximum_age_options
));

$seeking_relationship_label = elgg_echo('mfp:search:for');
// $seeking_relationship_type = mfp_get_profile_field_attr('seeking_relationship', 'type');
$seeking_relationship_options = mfp_get_profile_field_attr('seeking_relationship', 'options');
$seeking_relationship = elgg_view("input/multiselect", array(
	'name' => 'seeking_relationship',
	'value' => $defaults['seeking_relationship'],
	'options' => $seeking_relationship_options
));

$state_nation_label = elgg_echo('mfp:search:state_nation');
$state_nation_options = mfp_get_profile_field_attr('state_nation', 'options');
$state_nation = elgg_view("input/multiselect", array(
	'name' => 'state_nation',
	'value' => $defaults['state_nation'],
	'options' => $state_nation_options
));

$radius_options = mfp_get_profile_field_attr('match_maximum_distance', 'options');
$radius = elgg_view("input/dropdown", array(
	'name' => 'match_maximum_distance',
	'value' => $defaults['match_maximum_distance'],
	'options' => $radius_options
));

$zip_code_options = mfp_get_profile_field_attr('zip_code', 'options');
$zip_input = 'text';
if (elgg_is_active_plugin('mfp_inputs')) {
	$zip_input = 'pm_zipcode';
}
$zip_code = elgg_view("input/" . $zip_input, array(
	'name' => 'zip_code',
	'value' => $defaults['zip_code'],
	'options' => $zip_code_options,
	'placeholder' => elgg_echo('mfp:search:zip_code'),
	'class' => 'mfp_10_chars zipcode',
	'pattern' => '\d{5}?\\',
	'#class' => 'zipcode-input'
));

$zip_search = elgg_echo('mfp:search:zip_search', array($radius, $zip_code));
$zip_search_help = elgg_view('output/longtext', [
	'value' => elgg_echo('mfp:search:zip_search:help'),
	'class' => 'elgg-text-help'
]);

$pics_label = elgg_echo('mfp:search:pics_only');
if ($defaults['pics_only']) {
	$checked = 'checked = "checked"';
} else {
	$checked = '';
}
$pics = "<input type=\"checkbox\" name=\"pics_only\" id=\"pics_only\" value=\"1\" $checked />";

$sort_label = elgg_echo('mfp:search:sort');
$sort = elgg_view('input/dropdown', array(
	'name' => 'sort',
	'options_values' => array(
		'newest' => elgg_echo('mfp:search:newest_users'),
		'last_visit' => elgg_echo('mfp:search:last_visit'),
//		'online' => elgg_echo('mfp:search:online')
	),
	'value' => $defaults['sort']
));

$text_label = elgg_echo('mfp:search:text');
$text = elgg_view('input/text', array(
	'name' => 'text',
	'value' => $defaults['text'],
	'class' => 'mfp_10_chars'
));

// $submit = elgg_view('input/submit', array(
//	'value' => elgg_echo('mfp:search:submit')
// ));

$body = <<<___HTML
<div>
	<label for="gender">$gender_label</label>
	$gender
</div>

<div>
	<label for="seeking_gender">$seeking_gender_label</label>
	$seeking_gender
</div>

<div>
	<label for="match_minimum_age">$age_label</label>
	$match_minimum_age
	-
	<label for="match_maximum_age">$match_maximum_age</label>
</div>

<div>
	<label for="seeking_relationship">$seeking_relationship_label</label>
	$seeking_relationship
</div>

<div>
	<label for="state_nation">$state_nation_label</label>
	$state_nation
</div>

<div>
	$zip_search
	$zip_search_help
</div>

<div>
	<label for="pics_only">$pics_label</label>
	$pics
</div>

<div>
	<label for="sort">$sort_label</label>
	$sort
</div>

<div>
	<label for="text">$text_label</label>
	$text
</div>

<input type="hidden" name="mfp_profile_search" value="1" />

<div>
<button type="submit" class="elgg-button elgg-button-submit">Find profiles</button>
<button type="submit" class="elgg-button elgg-button-submit" name="mfp_matches" value="1">Load matches</button>
</div>

___HTML;

echo $body;

