<?php
/**
 * Base widget to show a set of matches
 *
 * If $vars['entities'] is populated use that. Else, search.
 * "Populated" means "anything that !== false"
 *
 */
$user = elgg_get_page_owner_entity();
$max_display = $vars['entity']->max_display;

if (!$max_display) {
	$max_display = 5;
}

$entities = isset($vars['entities']) ? $vars['entities'] : false;
$count = isset($vars['count']) ? $vars['count'] : 0;
$sort = isset($vars['sort']) ? $vars['sort'] : 'newest';
$more_link = isset($vars['more_link']) ? $vars['more_link'] : '';

if ($entities === false) {
	// set input to fake a search so we can reuse the search hook.
	// a few params we need to set manually
	set_input('limit', $max_display);
	set_input('sort', $sort);
	$params = array();
	// need to specify this for the page handler overrides
	$params['mfp_profile_search'] = 1;
	$params['sort'] = $sort;

	// some we can detect
	$search_fields = array(
		'gender',
		'seeking_gender',
		'match_minimum_age',
		'match_maximum_age',
		'seeking_relationship',
//		'state_nation',
		'zip_code',
		'match_maximum_distance'
	);

	if ($user) {
		$search_defaults = array();
		foreach ($search_fields as $name) {
			$value = $user->$name;
			if (!$value || empty($value)) {
				$value = null;
			}
			set_input($name, $value);
			$params[$name] = $value;
		}
		if (is_array($params['seeking_gender'])) {
			$params['seeking_gender'] = $params['seeking_gender'][0];
		}
		if (!is_array($params['seeking_relationship'])) {
			$params['seeking_relationship'] = array($params['seeking_relationship']);
		}
	}

	$results = elgg_trigger_plugin_hook('search', 'mfp_search', $params, []);

	$entities = $results['entities'];
	$count = $results['count'];
	$query = http_build_query($params);

	$url = elgg_get_site_url() . "search/mfp_search?$query";
	$more_link = "<a href=\"$url\">" . elgg_echo('more') . '</a>';
}

if ($count > 0) {
	$html = '';

	foreach ($entities as $entity) {
		$html .= elgg_view('search/mfp_search/entity', array('widget_view' => true, 'entity' => $entity));
	}

	// add more link
	if ($count > $max_display) {
		$html .= $more_link;
	}
} else {
	$html = elgg_echo('mfp_search:no_matches');
}

echo $html;
